@@ -24,7 +24,7 @@ except ImportError as e:
 try:
     import matplotlib.pyplot as plt
     AVAILABLE_LIBRARIES.add('matplotlib')
-    DEFAULT_MATPLOTLIB_COLORS = plt.rcParams['axes.prop_cycle'].by_key()['color'][1:] # Uncomment to remove blue color
+    DEFAULT_MATPLOTLIB_COLORS = plt.rcParams['axes.prop_cycle'].by_key()['color']       # [1:]
 except ImportError as e:
     logger.info("Matplotlib not installed, consider installing it to get more plotting capabilities")
 if 'matplotlib' in AVAILABLE_LIBRARIES:
@@ -39,13 +39,15 @@ if 'matplotlib' in AVAILABLE_LIBRARIES:
     except ImportError as e:
         logger.info("Tikzplotlib not installed, consider installing it to get more plotting capabilities")
 
-DEFAULT_PLOTLY_COLORS = ['rgb(31, 119, 180)',  # Blue
-                         'rgb(255, 127, 14)',  # Orange
-                         'rgb(44, 160, 44)',  # Green
-                         'rgb(214, 39, 40)',
-                         'rgb(148, 103, 189)', 'rgb(140, 86, 75)',
-                         'rgb(227, 119, 194)', 'rgb(127, 127, 127)',
-                         'rgb(188, 189, 34)', 'rgb(23, 190, 207)']
+DEFAULT_PLOTLY_COLORS = [
+    'rgb(31, 119, 180)',  # Blue
+    'rgb(255, 127, 14)',  # Orange
+    'rgb(44, 160, 44)',  # Green
+    'rgb(214, 39, 40)',
+    'rgb(148, 103, 189)', 'rgb(140, 86, 75)',
+    'rgb(227, 119, 194)', 'rgb(127, 127, 127)',
+    'rgb(188, 189, 34)', 'rgb(23, 190, 207)'
+]
 
 
 @dataclass
@@ -256,7 +258,7 @@ def get_confusion_matrix(y_pred_np, y_label_np, filename=None, axis_legends=None
     y_label_pd = pd.Series(y_label_np, name='Actual')
     df = pd.crosstab(y_label_pd, y_pred_pd, rownames=['Actual'], colnames=['Predicted'], margins=True, dropna=False)
     if filename is not None:
-        df.to_csv(filename)
+        df.to_csv(filename, index=False)
     # plot_confusion_matrix(df, filename, library='plotly', axis_legends=axis_legends)
     return df.fillna(0)
 
@@ -301,6 +303,7 @@ class SeveralMonteCarloComparison:
             NOTE: x and paths must be the same size
         """
         self.x_label = label
+        x = [str(x_tmp) for x_tmp in x]
         if all([item.isdigit() for item in x]):
             self.x = list(map(int, x))
         elif all([item.replace(".", "", 1).isdigit() for item in x]):
@@ -476,7 +479,7 @@ class SeveralMonteCarloComparison:
                 filter = [a == net and b == epoch for a, b in zip(df.network, df.epoch)]
                 data = df[filter].describe()
                 data = data[cols]
-                data.to_csv(path + net + "_" + self.x[i] + "_stats.csv")
+                data.to_csv(path + net + "_" + self.x[i] + "_stats.csv", index=False)
 
     def plot_histogram(self, key='accuracy', epoch=-1, library='seaborn', showfig=False, savefig=True, title='',
                        extension=".svg"):
@@ -506,7 +509,7 @@ class Plotter:
         self.file_suffix = file_suffix
         self.model_name = model_name
         self.pandas_dict = {}
-        
+
         if data_results_dict:
             result_pandas = pd.DataFrame.from_dict(data_results_dict)
             result_pandas.to_csv(self.path / f"{model_name}{file_suffix}", index=False)
@@ -524,15 +527,19 @@ class Plotter:
         # So ordering the files makes sure I open the Complex model first and so it plots with the same colours.
         # TODO: Think a better way without loosing generality (This sort is all done because of the ComplexVsReal case)
         for path, subdirs, files in os.walk(self.path):
-            files.sort()  # Respect the colors for the plot of Monte Carlo.
+            # files.sort()  # Respect the colors for the plot of Monte Carlo.
+            data_files = []
             for file in files:
                 if file.endswith(self.file_suffix):
-                    label = re.sub(self.file_suffix + '$', '', file).replace('_', ' ')
-                    tmp_df = pd.read_csv(Path(path) / file)
-                    if self.pandas_dict.get(label) is None:
-                        self.pandas_dict[label] = tmp_df
-                    else:
-                        self.pandas_dict[label] = pd.concat([self.pandas_dict[label], tmp_df], ignore_index=True)
+                    data_files.append(file)
+            # set_trace()
+            for file in data_files:
+                label = re.sub(self.file_suffix + '$', '', file).replace('_', ' ')
+                tmp_df = pd.read_csv(Path(path) / file)
+                if self.pandas_dict.get(label) is None:
+                    self.pandas_dict[label] = tmp_df
+                else:
+                    self.pandas_dict[label] = pd.concat([self.pandas_dict[label], tmp_df], ignore_index=True)
         # set_trace()
 
     def reload_data(self):
@@ -646,10 +653,10 @@ class Plotter:
         ax.set_ylabel(key)
         ax.set_xlabel("epoch")
         ax.set_title(title)
-        if showfig:
-            fig.show()
         if savefig:
             fig.savefig(str(self.path / key) + "_matplotlib" + extension, transparent=True)
+        if showfig:
+            fig.show()
 
     def _plot_plotly(self, key='loss', showfig=False, savefig=True, func=min, index_loc=None):
         if 'plotly' not in AVAILABLE_LIBRARIES:
@@ -755,7 +762,7 @@ class MonteCarloPlotter(Plotter):
         super().plot_key(key, reload, library, showfig, savefig, index_loc, extension=extension)
 
     def plot_line_confidence_interval(self, key='accuracy', showfig=False, savefig=True, library='matplotlib',
-                                      title='', full_border=True, x_axis='epoch', extension=".svg"):
+                                      title='', full_border=True, x_axis='epoch', extension=".svg", network_filter=None):
         """
         Saves/shows the evolution of the trainig median and mean with the corresponding confidence intervals
             of the results.
@@ -776,20 +783,23 @@ class MonteCarloPlotter(Plotter):
             self._plot_line_confidance_interval_plotly(key=key, showfig=showfig, savefig=savefig,
                                                        title=title, full_border=full_border, x_axis=x_axis)
         elif library == 'matplotlib' or library == 'seaborn':
-            self._plot_line_confidence_interval_matplotlib(key=key, showfig=showfig, savefig=savefig,
-                                                           title=title, x_axis=x_axis, extension=extension)
+            self._plot_line_confidence_interval_matplotlib(key=key, showfig=showfig, savefig=savefig, title=title,
+                                                           x_axis=x_axis, extension=extension,
+                                                           network_filter=network_filter)
         else:
             logger.warning("Warning: Unrecognized library to plot " + library)
             return None
 
     def _plot_line_confidence_interval_matplotlib(self, key='accuracy', showfig=False, savefig=True,
-                                                  title='', x_axis='epoch', extension=".svg"):
+                                                  title='', x_axis='epoch', extension=".svg", network_filter=None):
         if 'matplotlib' not in AVAILABLE_LIBRARIES:
             logger.warning("No Matplotlib installed, function " +
                            self._plot_line_confidence_interval_matplotlib.__name__ + " was called but will be omitted")
             return None
         fig, ax = plt.subplots()
         for i, (label, data) in enumerate(self.pandas_dict.items()):
+            if network_filter is not None and label.replace(' ', '_') not in network_filter:
+                continue
             x = data[x_axis].unique().tolist()
             data_mean = data[data['stats'] == 'mean'][key].tolist()
             if len(x) == len(data_mean):
@@ -823,6 +833,8 @@ class MonteCarloPlotter(Plotter):
             ax.fill_between(x, data_min, data_max, color=DEFAULT_MATPLOTLIB_COLORS[i], alpha=.15,
                             label=label.replace('_', ' ') + ' border')
         for label in self.pandas_dict.keys():
+            if network_filter is not None and label not in network_filter:
+                continue
             title += label.replace('_', ' ') + ' vs '
         title = title[:-3] + key
 
@@ -831,8 +843,6 @@ class MonteCarloPlotter(Plotter):
         ax.set_ylabel(key)
         ax.grid()
         ax.legend()
-        if showfig:
-            fig.show()
         if savefig:
             os.makedirs(str(self.path / "plots/lines_confidence/"), exist_ok=True)
             fig.savefig(self.path / ("plots/lines_confidence/montecarlo_" +
@@ -845,6 +855,8 @@ class MonteCarloPlotter(Plotter):
                 tikzplotlib.save(self.path / ("plots/lines_confidence/montecarlo_" +
                                               key.replace(" ", "_") + "_matplotlib" + ".tex"))
         # set_trace()
+        if showfig:
+            fig.show()
 
     def _plot_line_confidance_interval_plotly(self, key='accuracy', showfig=False, savefig=True,
                                               title='', full_border=True, x_axis='epoch'):
@@ -995,13 +1007,13 @@ class MonteCarloAnalyzer:
         if path is not None and df is not None:  # I have data and the place where I want to save it
             self.df = df  # DataFrame with all the data
             self.path = Path(path)
-            self.df.to_csv(self.path / "run_data.csv")  # Save the results for latter use
+            self.df.to_csv(self.path / "run_data.csv", index=False)  # Save the results for latter use
         elif path is not None and df is None:  # Load df from Path
             if not path.endswith("run_data.csv") or not path.endswith("run_data"): 
                 self.df = pd.DataFrame()
                 data_files = get_data_files_list(path)
                 for file in data_files:
-                    tmp_df = pd.read_csv(Path(file))
+                    tmp_df = pd.read_csv(Path(file), index_col=False)
                     # filter = [e == max(tmp_df.epoch) and n == 'complex_model' for e, n in
                     #           zip(tmp_df.epoch, tmp_df.network)]
                     # th = 0.3
@@ -1011,12 +1023,12 @@ class MonteCarloAnalyzer:
             else:
                 if not path.endswith('.csv'):
                     path += '.csv'
-                self.df = pd.read_csv(Path(path))  # Path(__file__).parents[1].absolute() /
+                self.df = pd.read_csv(Path(path), index_col=False)  # Path(__file__).parents[1].absolute() /
                 self.path = Path(os.path.split(path)[0])  # Keep only the path and not the filename
         elif path is None and df is not None:  # Save df into default path
             self.path = create_folder("./log/montecarlo/")
             self.df = df  # DataFrame with all the data
-            self.df.to_csv(self.path / "run_data.csv")  # Save the results for latter use
+            self.df.to_csv(self.path / "run_data.csv", index=False)  # Save the results for latter use
         else:  # I have nothing
             self.path = create_folder("./log/montecarlo/")
             self.df = pd.DataFrame()
@@ -1026,7 +1038,7 @@ class MonteCarloAnalyzer:
 
     def set_df(self, df, conf_mat=None):
         self.df = df  # DataFrame with all the data
-        self.df.to_csv(self.path / "run_data.csv")  # Save the results for latter use
+        self.df.to_csv(self.path / "run_data.csv", index=False)  # Save the results for latter use
         if conf_mat is not None:
             for i in range(len(conf_mat)):
                 mat = conf_mat[i]["matrix"]
@@ -1041,7 +1053,7 @@ class MonteCarloAnalyzer:
         networks_availables = self.df.network.unique()
         for net in networks_availables:
             data = self.df[self.df.network == net]
-            cols = ['loss', 'val_loss', 'accuracy', 'val_accuracy']
+            cols = [c for c in data.columns.values.tolist() if c not in ['network', 'epoch', 'path']]
             frames = []
             keys = []
             for epoch in data.epoch.unique():
@@ -1054,30 +1066,30 @@ class MonteCarloAnalyzer:
         # Save confusion matrix
         for i in range(len(self.confusion_matrix)):
             self.confusion_matrix[i]["matrix"].to_csv(self.path / (self.confusion_matrix[i]["name"]
-                                                                   + "_confusion_matrix.csv"))
+                                                                   + "_confusion_matrix.csv"), index=False)
 
     # ------------
     # Plot methods
     # ------------
 
-    def do_all(self, extension=".svg", showfig=False, savefig=True):
+    def do_all(self, extension=".svg", showfig=False, savefig=True, network_filter=None):
         """
         Plots box plot, histogram and confidence interval (using :code:`MonteCarloPlotter`) for both
             1. plotly
             2. seaborn
         And for keys:
-            1. val_accuracy
-            2. val_loss
+            1. val_accuracy (if available)
+            2. val_loss     (if available)
             3. accuracy
             4. loss
         """
-
-        key_list = ['accuracy', 'loss', 'val_accuracy', 'val_loss']
+        key_list = [c for c in self.df.columns.values.tolist() if c not in ['network', 'epoch', 'path', 'Unnamed: 0']]
         for key in key_list:
             # self.plot_3d_hist(key=key)
             for lib in ['seaborn', 'plotly']:
                 try:
-                    self.box_plot(key=key, extension=extension, library=lib, showfig=showfig, savefig=savefig)
+                    self.box_plot(key=key, extension=extension, library=lib, showfig=showfig, savefig=savefig,
+                                  network_filter=network_filter)
                 except:
                     logger.warning("Could not plot " + key + " Histogram with " + str(lib), exc_info=True)
                 try:
@@ -1088,12 +1100,13 @@ class MonteCarloAnalyzer:
                     logger.warning("Could not plot " + key + " Histogram with " + str(lib), exc_info=True)
                 try:
                     self.monte_carlo_plotter.plot_line_confidence_interval(key=key, x_axis='epoch', library=lib,
-                                                                           showfig=showfig, savefig=savefig)
+                                                                           showfig=showfig, savefig=savefig,
+                                                                           network_filter=network_filter)
                 except:
                     logger.warning(f"Could not plot {key} line_confidence_interval with {lib}", exc_info=True)
 
     def box_plot(self, epoch: int = -1, library: str = 'plotly', key: str = 'val_accuracy',
-                 showfig: bool = False, savefig: bool = True, extension: str = '.svg'):
+                 showfig: bool = False, savefig: bool = True, extension: str = '.svg', network_filter=None):
         """
         Saves/shows a box plot of the results.
         :param epoch: Which epoch to use for the box plot. If -1 (default) it will use the last epoch.
@@ -1109,7 +1122,8 @@ class MonteCarloAnalyzer:
         if library == 'plotly':
             self._box_plot_plotly(key=key, epoch=epoch, showfig=showfig, savefig=savefig)
         elif library == 'seaborn':
-            self._box_plot_seaborn(key=key, epoch=epoch, showfig=showfig, savefig=savefig, extension=extension)
+            self._box_plot_seaborn(key=key, epoch=epoch, showfig=showfig, savefig=savefig, extension=extension,
+                                   network_filter=network_filter)
         else:
             logger.warning("Warning: Unrecognized library to plot " + library)
             return None
@@ -1122,9 +1136,8 @@ class MonteCarloAnalyzer:
         fig = go.Figure()
         if epoch == -1:
             epoch = max(self.df.epoch)
-        networks_availables = self.df.network.unique()
-        # set_trace()
-        for col, net in enumerate(networks_availables):
+        networks_available = self.df.network.unique()
+        for col, net in enumerate(networks_available):
             filter = [a == net and b == epoch for a, b in zip(self.df.network, self.df.epoch)]
             data = self.df[filter]
             fig.add_trace(go.Box(
@@ -1164,8 +1177,11 @@ class MonteCarloAnalyzer:
             #                                 + "_box_plot" + extension)))
         elif showfig:
             fig.show(config=PLOTLY_CONFIG)
+        else:
+            plt.close(fig)
 
-    def _box_plot_seaborn(self, epoch=-1, key='val_accuracy', showfig=False, savefig=True, extension='.svg'):
+    def _box_plot_seaborn(self, epoch=-1, key='val_accuracy', showfig=False, savefig=True, extension='.svg',
+                          network_filter=None):
         if 'seaborn' not in AVAILABLE_LIBRARIES:
             logger.warning("No Seaborn installed, function " + self._box_plot_seaborn.__name__ +
                            " was called but will be omitted")
@@ -1175,7 +1191,9 @@ class MonteCarloAnalyzer:
         # Prepare data
         filter = self.df['epoch'] == epoch
         data = self.df[filter]
-
+        if network_filter:
+            filter = data['network'].isin(network_filter)
+            data = data[filter]
         # Run figure
         fig = plt.figure()
         ax = sns.boxplot(x="network", y=key, data=data, boxprops=dict(alpha=.3))
@@ -1200,6 +1218,8 @@ class MonteCarloAnalyzer:
                 tikzplotlib.save(filename + ".tex")
         if showfig:
             fig.show()
+        else:
+            plt.close(fig)
         return fig, ax
 
     def show_plotly_table(self):
@@ -1327,6 +1347,10 @@ class MonteCarloAnalyzer:
                    filename=self.path / (
                            "plots/histogram/montecarlo_" + key.replace(" ", "_") + "_matplotlib" + extension),
                    showfig=showfig, savefig=savefig)
+        if showfig:
+            fig.show()
+        else:
+            plt.close(fig)
         return fig, ax
 
     def _plot_histogram_plotly(self, key='val_accuracy', epoch=-1, showfig=False, savefig=True, title=''):
@@ -1366,6 +1390,8 @@ class MonteCarloAnalyzer:
             #                                  + "plotly_histogram" + extension)))
         elif showfig:
             fig.show(config=PLOTLY_CONFIG)
+        else:
+            plt.close(fig)
         return fig
 
     def _plot_histogram_seaborn(self, key='val_accuracy', epoch=-1,
@@ -1401,16 +1427,26 @@ class MonteCarloAnalyzer:
                 " will not save tex file")
         else:
             tikzplotlib.save(self.path / ("plots/histogram/montecarlo_" + key.replace(" ", "_") + "_seaborn" + ".tex"))
+        if showfig:
+            fig.show()
+        else:
+            plt.close(fig)
         return fig, ax
 
 
 if __name__ == "__main__":
-    path = "/home/barrachina/Documents/onera/src/PolSar/Oberpfaffenhofen/log/montecarlo/2021/02February/21Sunday/run-13h36m02"
+    path = "/mnt/point_de_montage/log/montecarlo/2021/04April/01Thursday/run-22h45m49"
     monte = MonteCarloAnalyzer(path=path)
     # monte.monte_carlo_plotter.plot_line_confidence_interval(showfig=True, library="plotly")
-    monte.plot_histogram(library="seaborn", showfig=True)
-    # monte.do_all(showfig=False, savefig=True)
+    # monte.plot_histogram(library="seaborn", showfig=True)
+    # monte.box_plot(library='seaborn', network_filter=['complex_network', 'real_network'])
+    monte.do_all(showfig=False, savefig=True, network_filter=['complex_network', 'real_network'])
     # set_trace()
+    # monte.monte_carlo_plotter.plot_line_confidence_interval()
+    #     key='accuracy', showfig=False, savefig=True,
+    #                                                         library='seaborn', title='', full_border=True,
+    #                                                         x_axis='epoch', extension=".svg",
+    #                                                         network_filter=['complex_network', 'real_network'])
     # monte.plot_histogram(library='matplotlib', showfig=False, savefig=True)
     # monte.monte_carlo_plotter.plot_train_vs_test(showfig=False, savefig=True)
     # monte.monte_carlo_plotter.plot_train_vs_test(showfig=False, savefig=True, key='loss', median=True)
@@ -1432,6 +1468,6 @@ if __name__ == "__main__":
     """
 
 __author__ = 'J. Agustin BARRACHINA'
-__version__ = '0.1.41'
+__version__ = '0.1.48'
 __maintainer__ = 'J. Agustin BARRACHINA'
 __email__ = 'joseagustin.barra@gmail.com; jose-agustin.barrachina@centralesupelec.fr'
