@@ -9,11 +9,13 @@ from abc import ABC, abstractmethod
 import plotly.graph_objects as go
 import plotly
 from matplotlib import pyplot as plt
+import matplotlib.colors as mcolors
 from scipy.linalg import eigh, cholesky
 from scipy.stats import norm
 import tikzplotlib
 
-MARKERS = [".", "x", "s", "+", "^", "D", "_", "v", "|", "*", "H"]
+MARKERS = ["*", "s", "x", "+", "^", "D", "_", "v", "|", ".", "H"]
+COLORS = list(mcolors.BASE_COLORS)
 logger = logging.getLogger(cvnn.__name__)
 
 
@@ -194,7 +196,11 @@ class Dataset:
                 for index, label in enumerate(labels):
                     if label == cls:
                         ax.plot(np.real(self.x[index]),
-                                np.imag(self.x[index]), MARKERS[cls % len(MARKERS)], label="Class " + str(cls))
+                                np.imag(self.x[index]),
+                                MARKERS[cls % len(MARKERS)],
+                                color=COLORS[cls % len(COLORS)],
+                                label="Class " + str(cls)
+                                )
                         ax.axis('equal')
                         ax.grid(True)
                         ax.set_aspect('equal', adjustable='box')
@@ -216,12 +222,13 @@ class Dataset:
         if showfig:
             fig.show()
         if save_path is not None:
+            save_path = Path(save_path)
             os.makedirs(save_path, exist_ok=True)
             prefix = ""
             if overlapped:
                 prefix = "overlapped_"
-            fig.savefig(save_path / Path(prefix + "data_example" + extension), transparent=True)
-            tikzplotlib.save(save_path / (prefix + "data_example.tikz"))
+            fig.savefig(save_path / Path(prefix + self.dataset_name + "_data_example" + extension), transparent=True)
+            tikzplotlib.save(save_path / (prefix + self.dataset_name + "_data_example.tikz"))
         return fig, ax
 
     # =======
@@ -363,7 +370,9 @@ class GeneratorDataset(ABC, Dataset):
 
 class CorrelatedGaussianNormal(GeneratorDataset):
 
-    def __init__(self, m, n, cov_matrix_list, num_classes=None, ratio=0.8, debug=False, savedata=False):
+    def __init__(self, m, n, cov_matrix_list, num_classes=None, ratio=0.8, debug=False, savedata=False,
+                 dataset_name=None, sort: bool = False):
+        self.sort = sort
         if num_classes is None:
             num_classes = len(cov_matrix_list)
         if not len(cov_matrix_list) == num_classes:
@@ -382,11 +391,13 @@ class CorrelatedGaussianNormal(GeneratorDataset):
                 logger.error("corelation coefficient module must be lower than one")
                 sys.exit(-1)
         self.cov_matrix_list = cov_matrix_list
+        if dataset_name is None:
+            dataset_name = "Correlated Gaussian Normal"
         super().__init__(m, n, num_classes=num_classes, ratio=ratio, savedata=savedata, debug=debug,
-                         dataset_name="Correlated Gaussian Normal")
+                         dataset_name=dataset_name)
 
     @staticmethod
-    def _create_correlated_gaussian_point(num_samples, r=None):
+    def _create_correlated_gaussian_point(num_samples, r=None, sort=False):
         # https: // scipy - cookbook.readthedocs.io / items / CorrelatedRandomSamples.html
         # Choice of cholesky or eigenvector method.
         method = 'cholesky'
@@ -412,10 +423,15 @@ class CorrelatedGaussianNormal(GeneratorDataset):
             evals, evecs = eigh(r)
             # Construct c, so c*c^T = r.
             c = np.dot(evecs, np.diag(np.sqrt(evals)))
-
         # Convert the data to correlated random variables.
         y = np.dot(c, x)
-        return [y[0][i] + 1j * y[1][i] for i in range(y.shape[1])]
+        y = [y[0][i] + 1j * y[1][i] for i in range(y.shape[1])]
+        if sort:
+            y.sort(key=lambda x: np.abs(x))
+        # tmp = [np.abs(y[i]) < np.abs(y[i+1]) for i in range(0, len(y)-1)]
+        # assert np.all(tmp)
+        # set_trace()
+        return y
 
     def _generate_data(self, num_samples_per_class, num_samples, num_classes):
         x = []
@@ -424,7 +440,7 @@ class CorrelatedGaussianNormal(GeneratorDataset):
             r = self.cov_matrix_list[signal_class]
             y.extend(signal_class * np.ones(num_samples_per_class))
             for _ in range(num_samples_per_class):
-                x.append(self._create_correlated_gaussian_point(num_samples, r))
+                x.append(self._create_correlated_gaussian_point(num_samples, r, sort=self.sort))
         return np.array(x), np.array(y)
 
     def summary(self, res_str=None):
@@ -477,7 +493,8 @@ class CorrelatedGaussianNormal(GeneratorDataset):
 
 class CorrelatedGaussianCoeffCorrel(CorrelatedGaussianNormal):
 
-    def __init__(self, m, n, param_list, num_classes=None, ratio=0.8, debug=False, savedata=False):
+    def __init__(self, m, n, param_list, num_classes=None, ratio=0.8, debug=False, savedata=False, dataset_name=None,
+                 sort: bool = False):
         if num_classes is None:
             num_classes = len(param_list)
         if not len(param_list) == num_classes:
@@ -492,7 +509,8 @@ class CorrelatedGaussianCoeffCorrel(CorrelatedGaussianNormal):
             sigma_xy = param[0] * sqrt(param[1] * param[2])
             cov_mat_list.append([[param[1], sigma_xy], [sigma_xy, param[2]]])
         super().__init__(m=m, n=n, cov_matrix_list=cov_mat_list,
-                         num_classes=num_classes, ratio=ratio, debug=debug, savedata=savedata)
+                         num_classes=num_classes, ratio=ratio, debug=debug, savedata=savedata,
+                         dataset_name=dataset_name, sort=sort)
 
 
 class ComplexNormalVariable(CorrelatedGaussianNormal):
@@ -639,13 +657,11 @@ def inf(a, b):
     return a > b
 
 
-def parametric_predictor(dataset, coef_1=0.5, coef_2=-0.5):
+def get_parametric_predictor_labels(x, y, coef_1=0.5, coef_2=-0.5):
     rho = []
-    x = np.real(dataset.x)
-    y = np.imag(dataset.x)
     for re, im in zip(x, y):
         cov = np.cov([re, im])
-        rho.append(cov[0][1] / (cov[0][0] * cov[1][1]))
+        rho.append(cov[0][1] / (np.sqrt(cov[0][0]) * np.sqrt(cov[1][1])))
 
     rho = np.array(rho)
     thresh = np.full(rho.shape, (coef_1 + coef_2) / 2)
@@ -653,7 +669,13 @@ def parametric_predictor(dataset, coef_1=0.5, coef_2=-0.5):
         result = np.less(rho, thresh).astype(int)
     else:
         result = np.greater(rho, thresh).astype(int)
+    return result
 
+
+def parametric_predictor(dataset, coef_1=0.5, coef_2=-0.5):
+    x = np.real(dataset.x)
+    y = np.imag(dataset.x)
+    result = get_parametric_predictor_labels(x=x, y=y, coef_1=coef_1, coef_2=coef_2)
     acc = np.sum(np.equal(Dataset.categorical_to_sparse(dataset.y), result)) / len(result)
     return acc
 
@@ -667,8 +689,13 @@ if __name__ == "__main__":
         [[1, -0.75], [-0.75, 1]]
     ]
     dataset = CorrelatedGaussianNormal(m, n, cov_matr_list, debug=False)"""
-    dataset = CorrelatedGaussianCoeffCorrel(m, n, param_list=[[0.5, 1, 1], [-0.5, 1, 1]])
-    dataset.save_data("./data/MLSP/")
+    for coef in [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]:
+        dataset = CorrelatedGaussianCoeffCorrel(m, n, param_list=[[coef, 1, 1], [-coef, 1, 1]],
+                                                dataset_name=f"{int(coef*100)}")
+        dataset.plot_data(overlapped=True, showfig=False,
+                          save_path="/home/barrachina/Dropbox/thesis/CVNN-thesis-Agustin/ppts/20210611 - ICASSP/img")
+    # dataset.save_data("./data/MLSP/")
+    # print(parametric_predictor(dataset))
 
     # dataset = OpenDataset("./data/MLSP/")
     # dataset.plot_data(overlapped=True, showfig=True, library="matplotlib")
@@ -676,6 +703,6 @@ if __name__ == "__main__":
     # print("{:.2%}".format(parametric_predictor(dataset)))
 
 __author__ = 'J. Agustin BARRACHINA'
-__version__ = '0.1.20'
+__version__ = '0.1.23'
 __maintainer__ = 'J. Agustin BARRACHINA'
 __email__ = 'joseagustin.barra@gmail.com; jose-agustin.barrachina@centralesupelec.fr'
